#ifndef DateiSystem_H
#define DateiSystem_H

#include <fstream>
#include "Array.h"
#include "Punkt.h"

namespace Framework
{
    class Bild; // Bild.h
    class Text; // Text.h
    class FBalken; // Fortschritt.h
#ifdef WIN32
    class Buchstabe; // Schrift.h
    class Alphabet; // Schrift.h
    class Schrift; // Schrift.h
#endif
    class LTDBPixel; // aus dieser Datei
    class LTDBKopf; // aus dieser Datei
    class LTDBBody; // aus dieser Datei
    class LTDBDatei; // aus dieser Datei
    class LTDSPixel; // aus dieser Datei
#ifdef WIN32
    class LTDSDateiKopf; // aus dieser Datei
    class LTDSSchriftKopf; // aus dieser Datei
    class LTDSBuchstabenKopf; // aus dieser Datei
    class LTDSBuchstabenKrper; // aus dieser Datei
    class LTDSDatei; // aus dieser Datei
#endif

    // LTDB Dateivormat --- Dient zum speichern von mehreren Bildern in einer Datei.

    // Dient zum Speichern und Laden eines einzelnen Pixels aus einem Bild im LTDB Dateiformat
    class LTDBPixel // Pixel einer LTDB Datei
    {
    private:
        LTDBPixel *davor; // Letzter Pixel
        char index; // Bitlnge des Pixels
        char iR, iG, iB, iA;
        char miR, miG, miB, miA;
        char maxIndex; // Lnge des Pixels
        bool change : 1; // Verndert sich etwas an den volgenden 5 Variablen
        bool changeR : 1; // ndert sich Rot
        bool changeG : 1; // ndert sich Grn
        bool changeB : 1; // ndert sich Blau
        bool changeA : 1; // ndert sich Alpha
        unsigned char komp : 3; // Komprimierung der Farbwerte
        unsigned char R; // Rot
        unsigned char G; // Grn
        unsigned char B; // Blau
        unsigned char A; // Alpha
        bool addBitZuFarbe( unsigned char bit ); // Fgt den Farbwerten ein Bit hinzu
        bool getNextFarbeBit( char &byte, int i ); // Speichert das nchste Farbbit in byte
        int ref;

    public:
        // Konstruktor
        //  davor: Der Pixel, der Vorher geladen wurde. 0, falls die der Erste Pixel ist
        __declspec( dllexport ) LTDBPixel( LTDBPixel *davor );
        // Destruktor 
        __declspec( dllexport ) ~LTDBPixel();
        // zum Laden gedacht. Fgt dem Pixel einiege bits hinzu
        //  byte: Das zuletzt aus der Datei gelesene Byte
        //  begin: Der Index des ersten Bits im byte, wo der Pixel beginnt
        //  return: Der Index des Bits im Byte, wo der Pixel aufgehhrt hat. -1, falls der Pixel am ende Des Bytes noch nicht zuende ist
        __declspec( dllexport ) char addByte( char byte, char begin );
        // zum speichern gedacht. Setzt die Farbe, die im Pixel gespeichert werden soll
        //  f: Die zu speichernde Farbe
        __declspec( dllexport ) void setFarbe( int f );
        // Komprimiert den Pixel. Muss vor dem Speichern aufgerufen werden.
        __declspec( dllexport ) void komprimieren();
        // Gibt ein Teil der Bits zurck, die den Pixel representieren
        //  byte: Eine Referens auf das Byte, dass als nchstes gespeichert werden soll
        //  begin: Der Index des ersten Bits im zu speichernden Byte, wo der Pixel gespeichert werden soll
        //  return: Der Index des Bits im zu speichernden Byte, wo der Pixel aufhhrt. -1, falls der Pixel im nchsten Byte fortgesetzt werden muss
        __declspec( dllexport ) char getNextByte( char &byte, int begin );
        // Gibt den Farbwert des Pixels zurck
        __declspec( dllexport ) int zuFarbe() const;
        // Gibt zurck, ob sich der Anteil an Rot in der Farbe im Vergleich zum Pixel davor gendert hat
        __declspec( dllexport ) bool getChangeR() const;
        // Gibt zurck, ob sich der Anteil an Grn in der Farbe im Vergleich zum Pixel davor gendert hat
        __declspec( dllexport ) bool getChangeG() const;
        // Gibt zurck, ob sich der Anteil an Blau in der Farbe im Vergleich zum Pixel davor gendert hat
        __declspec( dllexport ) bool getChangeB() const;
        // Gibt zurck, ob sich der Anteil an Alpha in der Farbe im Vergleich zum Pixel davor gendert hat
        __declspec( dllexport ) bool getChangeA() const;
        // Gibt die Komprimierung des Pixels zurck
        __declspec( dllexport ) unsigned char getKomp() const;
        // Gibt den Anteil an Rot in der Farbe des Pixels zurck
        __declspec( dllexport ) unsigned char getR() const;
        // Gibt den Anteil an Grn in der Farbe des Pixels zurck
        __declspec( dllexport ) unsigned char getG() const;
        // Gibt den Anteil an Blau in der Farbe des Pixels zurck
        __declspec( dllexport ) unsigned char getB() const;
        // Gibt den Anteil an Alpha in der Farbe des Pixels zurck
        __declspec( dllexport ) unsigned char getA() const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) LTDBPixel *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) LTDBPixel *release();
    };

    // Der Kopf des LTDB Dateiformates. Hier werden informationen ber alle Abgespeicherten Bilder verwaltet
    class LTDBDateiKopf
    {
    private:
        RCArray< Text > *bilder;
        Array< __int64 > *pos;
        int bAnzahl;
        int ref;

    public:
        // konstructor
        __declspec( dllexport ) LTDBDateiKopf();
        // destructor 
        __declspec( dllexport ) ~LTDBDateiKopf();
        // Entfernt ein Bild aus dem Dateikopf
        //  i: Der Index des Bildes, welches entfernt werden soll
        __declspec( dllexport ) void removeBild( int i );
        // Entfernt ein Bild aus dem Dateikopf
        //  txt: Der Name des Bildes, welches entfernt werden soll
        __declspec( dllexport ) void removeBild( Text *txt );
        // Fgt dem Dateikopf ein Bild hinzu
        //  txt: Der Name des Bildes.
        __declspec( dllexport ) void addBild( Text *txt );
        // Legt den Index des Bytes aus der Datei fest, wo das Bild anfngt
        //  i: Der index Des Bildes
        //  pos: Die Position des Bildes in der Datei
        __declspec( dllexport ) void setBildPos( int i, __int64 pos );
        // Legt den Index des Bytes aus der Datei fest, wo das Bild anfngt
        //  txt: Der Name Des Bildes
        //  pos: Die Position des Bildes in der Datei
        __declspec( dllexport ) void setBildPos( Text *txt, __int64 pos );
        // Ldt den Dateikopf einer LTDB Datei
        //  f: Ein Zeiger auf einen Fortschrittsbalken, der zum Laden verwendet werden soll. Kann 0 sein.
        //  inF: Der geffnete ifstream der LTDB Datei, bei dem die Leseposition bereits auf das erste Byte des Dateiopfes zeigt.
        __declspec( dllexport ) void laden( FBalken *f, std::ifstream *inF );
        // Speichert den Dateikopf in einer LTDB Datei
        //  outF: Der geffnete ofstream der LTDB Datei, bei dem die Schreibposition bereits auf das erste Byte des Dateikopfes zeigt.
        __declspec( dllexport ) void speichern( std::ofstream *outF ) const;
        // Gibt den Namen eines bestimmten Bildes zurck
        //  i: Der Index des Bildes, dessen Namen zurckgegeben werden soll
        //  return: Der Name des bildes
        __declspec( dllexport ) Text *getBild( int i ) const;
        // Gibt den Namen eines bestimmten Bildes zurck
        //  i: Der Index des Bildes, dessen Namen zurckgegeben werden soll
        //  return: Der Name des bildes ohne erhhten Reference Counter
        __declspec( dllexport ) Text *zBild( int i ) const;
        // Gibt den Index des Ersten Bytes eines Bildes in der Datei zurck
        //  txt: Der Name des Bildes, dessen Beginn gefunden werden soll
        //  return: -1, falls das Bild nicht gefunden wurde.
        __declspec( dllexport ) __int64 getBildPosition( Text *txt ) const;
        // Gibt den Index des Ersten Bytes eines Bildes in der Datei zurck
        // Fals der Index nicht existiert wird die Exception std::out_of_range geworfen.
        //  indes: Der Indes des Bildes, dessen Beginn gefunden werden soll
        __declspec( dllexport ) __int64 getBildPosition( int index ) const;
        // Gibt den Index eines Bestimmten Bildes zurck
        //  txt: Der Name des Bildes
        //  return: -1, falls das Bild nicht gefunden wurde
        __declspec( dllexport ) int getBildIndex( Text *txt ) const;
        // Gibt die Anzahl der Bilder in der Datei zurck
        __declspec( dllexport ) int getbAnzahl() const;
        // Gibt eine Liste mit Bildern in der Datei ohne erhhten Reference Counter zurck.
        // Die Liste sollte nicht verndert werden
        __declspec( dllexport ) RCArray< Text > *zBildListe() const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) LTDBDateiKopf *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) LTDBDateiKopf *release();
    };

    // Die im LTDB Dateikopf gespeicherten Informationen fr ein einzelnes Bild
    class LTDBKopf
    {
    private:
        __int64 a; // Det LTDB Dateikopf ist maximal 104 Bits lang
        __int32 b; // - bis zu 75 Bits fr den Titel
        __int8 c;  // - 12 Bits fr breite
        int ref;   // - 12 Bits fr hhe

    public:
        // Konstruktor 
        __declspec( dllexport ) LTDBKopf();
        // Lhdt die Informationen ber ein Bild
        //  f: Der geffnette und an die richtiege Stelle zeigende ifstream der LTDB Datei
        __declspec( dllexport ) void laden( std::ifstream *f );
        // Setzt die Informationen die gespeichert werden sollen
        //  titel: Der Titel des Bildes
        //  size: Die Gre des Bildes
        //  return: Die Anzahl der Buchstaben aus dem Titel, die im LTDB Dateiformat nicht gespeichert werden knnen. Erlaubt ist nur a-z und A-Z und     und    und .
        // Alle grobuchstaben im Titel werden in Kleinbuchstaben umgewandelt
        __declspec( dllexport ) int Init( Text *titel, const Punkt &size );
        // Lhd informationen aus geladenen Bits. Wird von der laden( std::ifstream ) Funktion verwendet.
        //  BeginBit: Der Index des ersten Bits, welches ausgewertet werden soll
        //  EndBit: Der Index des letzten Bits, welches nichtmehr ausgewertet werden soll
        //  bits: Die Bits, von denen alle von BeginBit bis EndBit ausgewertet werden sollen
        // Insgesamt mssen 104 Bits gesetzt werden. Hierauf bezihen sich BeginBit und EndBit
        __declspec( dllexport ) void setBits( int BeginBit, int EndBit, __int16 bits );
        // Speichert die Informationen in eine Datei
        //  f: Der geffnete und an die richtiege Stelle zeigende ofstream der LTDB Datei
        __declspec( dllexport ) void speichern( std::ofstream *f ) const;
        // Gibt die Lnge des Titels zurck
        __declspec( dllexport ) int getTitelLength() const;
        // Gibt den Titel des Bildes zurck
        __declspec( dllexport ) Text *getTitel() const;
        // Gibt die Gre des Bildes zurck
        __declspec( dllexport ) Punkt getSize() const;
        // Gibt die nchsten zu speichernden Bits zurck
        //  begin: Der Index des ersten Bits, in das gespeichert werden soll
        //  end: Der Index des letzten Bits, in das gespeichert werden soll
        //  return: 16 Bits, in denen die Informationen zwischen begin und end stehen
        // Insgesamt mssen 104 Bits gelesen werden. Hierauf bezihen sich BeginBit und EndBit
        __declspec( dllexport ) __int16 getBits( int begin, int end )const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) LTDBKopf *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) LTDBKopf *release();
    };

    // Verwaltet die Pixeldaten eines einzelnen Bildes einer LTDB Datei
    class LTDBBody
    {
    private:
        Punkt gr;
        Bild *b;
        int dateiSize;
        int ref;

    public:
        // Konstruktor 
        __declspec( dllexport ) LTDBBody();
        // Konstruktor
        //  k: Der LTDB Kopf des Bildes, der Informationen ber die Gre des Bildes enthlt
        __declspec( dllexport ) LTDBBody( LTDBKopf *k );
        // Destruktor 
        __declspec( dllexport ) ~LTDBBody();
        // Setzt die Informationen ber die Gre des Bildes. Wird zum Laden bentigt.
        //  k: Der LTDB Kopf des Bildes
        __declspec( dllexport ) void init( LTDBKopf k );
        // Setzt die Informationen ber die Gre des Bildes. Wird zum Laden bentigt.
        //  k: Der LTDB Kopf des Bildes
        __declspec( dllexport ) void init( LTDBKopf *k );
        // Ldt die Pixeldaten aus der Datei
        //  zF: Ein Fortschrittsbalken, der 0 sein kann
        //  inF: Der geffnete und an die richtiege Stelle zeigende ifstream der LTDB Datei
        __declspec( dllexport ) void laden( FBalken *zF, std::ifstream *inF );
        // Setzt das Bild, welches gespeichert werden soll
        //  b: Das zu speichernde Bild
        __declspec( dllexport ) void setBild( Bild *b );
        // Speichert die Pixeldaten des Bildes in einer LTDB Datei
        //  zF: Ein Fortschrittsbalken, der 0 sein kann
        //  outF: Der geffnete und an die richtiege Stelle zeigende ofstream der LTDB Datei
        __declspec( dllexport ) void speichern( FBalken *zF, std::ofstream *outF ) const;
        // Gibt das geladene Bild zurck
        __declspec( dllexport ) Bild *getBild() const;
        // Gibt die Gre des Bildes zurck
        __declspec( dllexport ) const Punkt &getSize() const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) LTDBBody *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) LTDBBody *release();
    };

    // Verwaltet eine LTDB Datei
    class LTDBDatei
    {
    private:
        Text *pfad;
        LTDBDateiKopf *datKpf;
        int ref;

    public:
        // Konstruktor 
        __declspec( dllexport ) LTDBDatei();
        // Destruktor 
        __declspec( dllexport ) ~LTDBDatei();
        // Setzt den Pfad zu Datei
        //  pfad: Der Pfad
        __declspec( dllexport ) void setDatei( Text *pfad );
        // Erstellt eine neue LTDB Datei
        __declspec( dllexport ) void erstellen();
        // Liet grundlegende Informationen aus der Datei.
        // Wird bentigt, bevor mit der Datei gearbeitet wird
        //  zF: Ein Fortschrittsbalken, der 0 sein kann
        __declspec( dllexport ) void leseDaten( FBalken *zF );
        // Lscht die LTDB Datei
        __declspec( dllexport ) void remove();
        // Lscht ein Bild aus der LTDB Datei
        //  zF: Ein Fortschrittsbalken der 0 sein kann
        //  name: Der Name des zu lschenden Bildes
        __declspec( dllexport ) void remove( FBalken *zF, Text *name );
        // Ldt ein Bild aus der LTDB Datei
        //  zF: Ein Fortschrittsbalken, der 0 sein kann
        //  name: Der Name des Bildes, welches geladen werden soll
        //  return: Das geladene Bild. 0, falls das Bild nicht gefunden wurde
        __declspec( dllexport ) Bild *laden( FBalken *zF, Text *name );
        // Speichert ein neues Bild in der LTDB Datei
        //  zF: Ein Fortscrittsbalken, der 0 sein kann
        //  bild: Das Bild, welches gelscht werden soll
        //  name: Der Name, unter dem das Bild gespeichert werden soll
        //  return: Anzahl der Warnungen, die beim Konvertieren des Namens in einen gltigen Namen aufgetreten sind. -1, falls bereis ein Bild mit dem selben Namen existiert
        __declspec( dllexport ) int speichern( FBalken *zF, Bild *bild, Text *name );
        // Gibt eine Liste mit gespeicherten Bildern zurck
        // Die Liste sollte nicht verndert werden
        __declspec( dllexport ) RCArray< Text > *zBildListe();
        // Gibt den Pfad zur LTDB Datei zurck
        __declspec( dllexport ) Text *getPfad() const;
        // Gibt die Anzahl der Bilder in der LTDB Datei zurck
        __declspec( dllexport ) int getBildAnzahl() const;
        // Prft, ob die LTDB Datei existiert
        __declspec( dllexport ) bool istOffen() const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) LTDBDatei *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) LTDBDatei *release();
    };

#ifdef WIN32
    // LTDS Dateivormat --- Dient zum speichern von Schriftarten

    // Verwaltet einen einzelnen Pixel von einem Buchstaben
    class LTDSPixel
    {
    private:
        int ref;
        char index;
        char iA;
        char miA;
        char maxIndex; // Lnge des Pixels
        bool nder;
        bool nderA;
        unsigned char komp : 3; // Komprimierung der Farbwerte
        unsigned char alpha;
        LTDSPixel *davor;
        bool addBitZuFarbe( unsigned char bit ); // Fgt den Farbwerten ein Bit hinzu
        bool getNextFarbeBit( char &byte, int i ); // Speichert das nchste Farbbit in byte

    public:
        // Konstruktor
        //  davor: Der Pixel, der vor diesem geladen wurde. 0, fals die der erste Pixel ist
        __declspec( dllexport ) LTDSPixel( LTDSPixel *davor );
        // Destruktor 
        __declspec( dllexport ) ~LTDSPixel();
        // Fgt dem Pixel einiege geladene Bits hinzu. Zum Laden gedacht.
        //  byte: Das letzte aus der Datei geladene Byte.
        //  begin: Der Index des ersten Bits im Byte, wo der Pixel beginnt
        //  return: Der Index des Letzten Bits in Byte, wo der Pixel aufhhrt. -1, falls der Pixel im nchsten Byte weitergeht
        __declspec( dllexport ) char addByte( char byte, char begin );
        // Setzt den Alpha Wert des Pixels. Zum speichern gedacht.
        //  alpha: Der Alpha Wert des Pixels.
        __declspec( dllexport ) void setAlpha( unsigned char alpha );
        // Komprimiert den Pixel. Muss vor dem Speichern aufgerufen werden.
        __declspec( dllexport ) void Komp();
        // Gibt ein Teil der Bits zurck, die den Pixel representieren
        //  byte: Eine Referens auf das Byte, dass als nchstes gespeichert werden soll
        //  begin: Der Index des ersten Bits im zu speichernden Byte, wo der Pixel gespeichert werden soll
        //  return: Der Index des Bits im zu speichernden Byte, wo der Pixel aufhhrt. -1, falls der Pixel im nchsten Byte fortgesetzt werden muss
        __declspec( dllexport ) char getNextByte( char &byte, int bbegin );
        // Gibt die Komprimierung des Pixels zurck
        __declspec( dllexport ) unsigned char getKomp() const;
        // Gibt zurck, ob sich der Alpha Wert im Vergleich zu dem Pixel davor gendert hat
        __declspec( dllexport ) bool getnderA() const;
        // Gibt den Alpha Wert des Pixels zurck
        __declspec( dllexport ) unsigned char getA() const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) LTDSPixel *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) LTDSPixel *release();
    };

    // Verwaltet den Kopf einer LTDS Datei. Er enthlt Informationen ber die in der Datei gespeicherten Schriftgren
    class LTDSDateiKopf
    {
    private:
        int ref;
        unsigned char sganzahl;
        unsigned char *gr;
        int *pos;

    public:
        // Konstruktor 
        __declspec( dllexport ) LTDSDateiKopf();
        // Destruktor 
        __declspec( dllexport ) ~LTDSDateiKopf();
        // Ldt den Kopf aus der LTDS Datei
        //  inF: Der geffnete und an die richtige Stelle zeigende ifstream der LTDS Datei
        __declspec( dllexport ) void laden( std::ifstream *inF );
        // Fgt eine Schriftgre hinzu
        //  sg: Die Schriftgre, die hinzugefgt werden soll
        __declspec( dllexport ) void addSG( char sg );
        // Lscht eine Schriftgre
        //  sg: Die Schriftgre, die gelscht werden soll
        __declspec( dllexport ) void removeSG( char sg );
        // Speichert den LTDS Kopf in der Datei
        //  outF: Der geffnete und an die richtiege Stelle zeigende ofstream der LTDS Datei
        __declspec( dllexport ) void speichern( std::ofstream *outF ) const;
        // Gibt einen Array von gespeicherten Schriftgren zurck
        // Der Array sollte nicht verdert werden
        __declspec( dllexport ) unsigned char *getSchriftGreList() const;
        // Gibt einen Array mit positionen der ersten Bytes fr jede Schriftgre aus der Datei zurck
        __declspec( dllexport ) int *getPositionList() const;
        // Gibt die Anzahl der gespeicherten Schriftgren zurck
        __declspec( dllexport ) int getSchriftGreAnzahl() const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) LTDSDateiKopf *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) LTDSDateiKopf *release();
    };

    // Der Kopf einer Schriftgre. Enthlt Informationen ber die gespeicherten Zeichen
    class LTDSSchriftKopf
    {
    private:
        int ref;
        unsigned char schriftSize;
        unsigned char *zeichen;
        int *pos;
        unsigned char zeichenAnzahl;

    public:
        // Konstruktor 
        __declspec( dllexport ) LTDSSchriftKopf();
        // Destruktor 
        __declspec( dllexport ) ~LTDSSchriftKopf();
        // Ldt den Kopf einer Schriftgre aus der Datei
        //  inF: Der geffnete und an de richtiege Stelle zeigende ifstream der LTDS Datei
        __declspec( dllexport ) void laden( std::ifstream *inF );
        // Setzt die Schriftgre. Zum speichern gedacht
        //  gr: Die Schriftgre
        __declspec( dllexport ) void setSchriftgre( unsigned char gr );
        // Setzt das Alphabet, welches in der Schriftgre gespeichert werden soll
        //  alphabet: Das Alphabet, welches alle zu speichernden Zeichen in der Schriftgre enthlt
        __declspec( dllexport ) void setZeichenAlphabet( Alphabet *alphabet );
        // Fgt der Schriftgre ein Zeichen hinzu, welches gespeichert werden soll
        //  zeichen: Der ASCII code des Zeichens, welches hinzugefgt werden soll
        __declspec( dllexport ) void addZeichen( unsigned char zeichen );
        // Lscht ein Zeichen aus der Schriftgre
        //  zeich: Der ASCII code des Zeichens, welches gelscht werden soll
        __declspec( dllexport ) void removeZeichen( unsigned char zeich );
        // Speichert den Kopf der Schriftgre in der LTDS Datei
        //  outF: Der geffnete und an die richtiege Stelle zeigende ofstream der LTDS Datei
        __declspec( dllexport ) void speichern( std::ofstream *outF ) const;
        // Gibt die Schriftgre zurck, zu der dieser Kopf gehhrt
        __declspec( dllexport ) unsigned char getSchriftGre() const;
        // Gibt die Anzahl der in der Schriftgre gespeicherten Zeichen zurck
        __declspec( dllexport ) unsigned char getZeichenAnzahl() const;
        // Gibt einen Array mit den Positionen der ersten Bytes von den gespeicherten Zeichen aus der LTDS Datei zurck 
        __declspec( dllexport ) int *getPositionen() const;
        // Gibt einen Array mit den ASCII codes der Gespeicherten Zeichen zurck
        __declspec( dllexport ) unsigned char *getZeichen() const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) LTDSSchriftKopf *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) LTDSSchriftKopf *release();
    };

    // Der Kopf eines einzelnen Zeichens aus der LTDS Datei. Enthlt informationen ber die Pixelgre des Zeichens
    class LTDSBuchstabenKopf
    {
    private:
        int ref;
        unsigned char zeichen;
        Punkt size;

    public:
        // Konstruktor 
        __declspec( dllexport ) LTDSBuchstabenKopf();
        // Ldt die Daten aus der LTDS Datei
        //  inF: Der geffnete und an die richtiege Stelle zeigende ifstream der LTDS Datei
        __declspec( dllexport ) void laden( std::ifstream *inF );
        // Setzt die Daten, die gespeichert werden sollen.
        //  zeichen: Der ASCII code des Zeichens
        //  gre: Die Gre des Zeichens in Pixeln
        __declspec( dllexport ) void init( unsigned char zeichen, const Punkt &gre );
        // Setzt die Daten, die gespeichert werden sollen.
        //  zeichen: Der ASCII code des Zeichens
        //  br: Die Breite des Zeichens in Pixeln
        //  h: Die Hhe des Zeichens in Pixeln
        __declspec( dllexport ) void init( unsigned char zeichen, int br, int h );
        // Speichert die Daten in der LTDS Datei
        //  outF: Der geffnete und auf die richtiege Stelle zeigende ofstream der LTDS Datei
        __declspec( dllexport ) void speichern( std::ofstream *outF ) const;
        // Gibt den ASCII code des Zeichens zurck
        __declspec( dllexport ) unsigned char getZeichen() const;
        // Gibt die Breite des Zeichens in Pixeln zurck
        __declspec( dllexport ) int getBreite() const;
        // Gibt die Hhe des Zeichens in Pixeln zurck
        __declspec( dllexport ) int getHhe() const;
        // Gib t die Gre des Zeichens in Pixeln zurck
        __declspec( dllexport ) const Punkt &getGre() const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) LTDSBuchstabenKopf *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) LTDSBuchstabenKopf *release();
    };

    // Verwaltet die Pixeldaten eines Zeichens
    class LTDSBuchstabenKrper
    {
    private:
        int ref;
        Punkt size;
        unsigned char zeichen;
        Buchstabe *buchstabe;

    public:
        // Konstruktor
        //  kopf: Der Kopf des Zeichens
        __declspec( dllexport ) LTDSBuchstabenKrper( LTDSBuchstabenKopf *kopf );
        // Destruktor 
        __declspec( dllexport ) ~LTDSBuchstabenKrper();
        // Setzt den Buchstaben, der gespeichert werden soll
        //  zeichen: Der zu speichernde Buchstabe
        __declspec( dllexport ) void setBuchstabe( Buchstabe *zeichen );
        // Ldt die Pixel aus der LTDS Datei
        //  inF: Der geffnete und auf die richtiege Stelle zeigende ifstream der LTDS Datei
        __declspec( dllexport ) void laden( std::ifstream *inF );
        // Speichert die Pixel in die LTDS Datei
        //  outF: der geffnete und an die richtiege Stelle zeigende ofstream der LTDS Datei
        __declspec( dllexport ) void speichern( std::ofstream *outF ) const;
        // Gibt den geladenen Buchstaben zurck
        __declspec( dllexport ) Buchstabe *getBuchstabe() const;
        // Gibt den ASCII code des Buchstabens zurck
        __declspec( dllexport ) unsigned char getZeichen() const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) LTDSBuchstabenKrper *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) LTDSBuchstabenKrper *release();
    };

    // Verwaltet eine LTDS Datei
    class LTDSDatei
    {
    private:
        int ref;
        Text *pfad;
        LTDSDateiKopf *dateiKopf;

    public:
        // Konstruktor 
        __declspec( dllexport ) LTDSDatei();
        // Destruktor 
        __declspec( dllexport ) ~LTDSDatei();
        // Setzt den Pfad zur Datei
        //  txt: Der Pfad
        __declspec( dllexport ) void setPfad( Text *txt );
        // Ldt wichtiege Informationen aus der Datei. Muss vor dem Verwenden der Datei aufgerufen werden
        __declspec( dllexport ) void leseDaten();
        // Fgt der Datei eine Schriftgre hinzu, fals diese noch nicht existiert
        //  alphabet: Das Alphabet, welches die Zeichen in der gewnschten Schriftgr0e enthlt
        __declspec( dllexport ) void addSchriftgre( Alphabet *alphabet );
        // Fgt einer Schriftgre einen Buchstaben hinzu
        //  gr: Die Schriftgre des Buchstabens
        //  zeich: Der Buchstabe, der gespeichert werden soll
        //  zeichen: Der ASCII code des Buchstabens
        __declspec( dllexport ) void addBuchstabe( int gr, Buchstabe *zeich, unsigned char zeichen );
        // Lscht eine bestimmte Schriftgre aus der Datei
        //  gr: Die Schriftgre, die entfernt werden soll
        __declspec( dllexport ) void lscheSchrifrGre( int gr );
        // Lscht einen Buchstaben aus einer Schriftgre
        //  gr: Die Schriftgre, aus der der Buchstabe entfernt werden soll
        //  zeichen: Der ASCII code des Zeichens, welches gelscht werden soll
        __declspec( dllexport ) void lscheBuchstabe( int gr, unsigned char zeichen );
        // Lscht die LTDS Datei
        __declspec( dllexport ) void lscheDatei();
        // Erstellt die LTDS Datei
        __declspec( dllexport ) void erstelleDatei();
        // Speichert eine gesammte Schrift in der Datei
        //  schrift: Die schrift, die gespeichert werden soll
        __declspec( dllexport ) void speicherSchrift( Schrift *schrift );
        // Ldt die gesammte Schrift aus der Datei
        //  return: Die geladene Schrift. 0, falls ein Fehler beim Laden aufgetreten ist
        __declspec( dllexport ) Schrift *ladeSchrift();
        // Ldt eine einzelne Schriftgre aus der Datei
        //  schriftgre: Die zu ladende Schriftgre
        //  return: Ein Alphabet mit den Zeichen in der Schriftgre. 0, falls die Schriftgre nicht gefunden wurde
        __declspec( dllexport ) Alphabet *ladeAlphabet( int schriftgre );
        // Ldt ein bestimmtes Zeichen einer bestimmten Schriftgre
        //  schriftgre: Die Schriftgre, zu dem das Zeichen gehrt
        //  zeichen: Der ASCII code des zu ladenden Zeichens
        //  return: Der geladene Buchstabe. 0, falls das Zeichen nicht gefunden wurde.
        __declspec( dllexport ) Buchstabe *ladeBuchstabe( int schriftgre, unsigned char zeichen );
        // Gibt den Pfad zur LTDS Datei zurck
        __declspec( dllexport ) Text *getPfad() const;
        // Gibt die Anzahl der gespeicherten Schriftgren zurck
        __declspec( dllexport ) int getAnzahlSchriftgren() const;
        // Gibt einen Array mit den gespeicherten Schriftgren zurck
        // Der Array sollte nicht verndert werden
        __declspec( dllexport ) unsigned char *getSchriftGren() const;
        // Gibt die Anzahl an gespeicherten Zeichen in einer Schriftgre zurck
        //  sg: Die Schriftgre, von der die Anzahl der ZEichen ermittelt werden soll
        //  return: Die Anzahl der Zeichen.
        __declspec( dllexport ) unsigned char getAnzahlBuchstaben( int sg );
        // Gibt einen Array mit Buchstaben einer bestimmten Schriftgre zurck
        //  sg: Die Schriftgre
        //  return: Der Array mit den ASCII codes der Zeichen. 0, falls die Schriftgre nicht gefunden wurde.
        __declspec( dllexport ) unsigned char *getBuchstaben( int sg );
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) LTDSDatei *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) LTDSDatei *release();
    };
#endif
    // Bit Funktionen 

    // gibt 1-bits in gewinschter anzahl zurck.
    //  a: Die Anzahl der Bits, die 1 sein sollen
    //  return: 32 Bits, wobei die einser Bits von rechts beginnen
    __declspec( dllexport ) int Bits( int a );
    // Gibt zurck, wie viele Bits bentigt werden um eine Zahl darzustellen
    //  c: Die Zahl, die dargestellt werden soll
    //  return: Die Anzahl der bentigten Bits
    __declspec( dllexport ) int getBits( char c );
}
#endif